/**
 * Creates the stylesheet content to be injected inside the head
 * of the document
 */
export declare function createStyleSheet(): string;
/**
 * Returns the script tag contents to be injected inside the head
 * of the document
 */
export declare function createScript(): string;
