declare const _default: ({
    match: RegExp;
    sub: string;
    expand?: undefined;
    type?: undefined;
} | {
    expand: string;
    match?: undefined;
    sub?: undefined;
    type?: undefined;
} | {
    type: string;
    match: RegExp;
    sub?: undefined;
    expand?: undefined;
})[];
export default _default;
//# sourceMappingURL=sql.d.ts.map