var b=Object.defineProperty;var n=Object.getOwnPropertyDescriptor;var o=Object.getOwnPropertyNames;var x=Object.prototype.hasOwnProperty;var d=(l,r)=>{for(var a in r)b(l,a,{get:r[a],enumerable:!0})},c=(l,r,a,t)=>{if(r&&typeof r=="object"||typeof r=="function")for(let m of o(r))!x.call(l,m)&&m!==a&&b(l,m,{get:()=>r[m],enumerable:!(t=n(r,m))||t.enumerable});return l};var g=l=>c(b({},"__esModule",{value:!0}),l);var y={};d(y,{default:()=>u});module.exports=g(y);var e={black:"\x1B[30m",red:"\x1B[31m",green:"\x1B[32m",gray:"\x1B[90m",yellow:"\x1B[33m",blue:"\x1B[34m",magenta:"\x1B[35m",cyan:"\x1B[36m",white:"\x1B[37m"};var u={deleted:e.red,var:e.red,err:e.red,kwd:e.magenta,num:e.yellow,class:e.yellow,cmnt:e.gray,insert:e.green,str:e.green,bool:e.cyan,type:e.blue,oper:e.blue,section:e.magenta,func:e.blue};
