import { notImplementedClass } from "../_internal/utils.mjs";
export const StringDecoder = globalThis.StringDecoder || /* @__PURE__ */ notImplementedClass("string_decoder.StringDecoder");
export default { StringDecoder };
