// src/workers/analytics-engine/analytics-engine.worker.ts
var LocalAnalyticsEngineDataset = class {
  constructor(env) {
    this.env = env;
  }
  writeDataPoint(_event) {
  }
};
function analytics_engine_worker_default(env) {
  return new LocalAnalyticsEngineDataset(env);
}
export {
  analytics_engine_worker_default as default
};
//# sourceMappingURL=analytics-engine.worker.js.map
