import{c as e,s as t,z as n}from"./useRouterState-DyVgzXuA.js";import{St as r}from"./button-cdxnqcgzwko8ooha-DePtDphh.js";var i=e(),a=n(t(),1),o={variant:{heading1:{classes:`text-3xl font-semibold`,description:`Large heading for page titles`},heading2:{classes:`text-2xl font-semibold`,description:`Medium heading for section titles`},heading3:{classes:`text-lg font-semibold`,description:`Small heading for subsections`},body:{classes:`text-kumo-default`,description:`Default body text`},secondary:{classes:`text-kumo-subtle`,description:`Muted text for secondary information`},success:{classes:`text-kumo-link`,description:`Success state text`},error:{classes:`text-kumo-danger`,description:`Error state text`},mono:{classes:`font-mono`,description:`Monospace text for code`},"mono-secondary":{classes:`font-mono text-kumo-subtle`,description:`Muted monospace text`}},size:{xs:{classes:`text-xs`,description:`Extra small text`},sm:{classes:`text-sm`,description:`Small text`},base:{classes:`text-base`,description:`Default text size`},lg:{classes:`text-lg`,description:`Large text`}}};function s({variant:e=`body`,bold:t=!1,size:n=`base`,truncate:s=!1,children:c,DANGEROUS_className:l,DANGEROUS_style:u,as:d,...f},p){let m=[`body`,`secondary`,`success`,`error`].includes(e),h=[`mono`,`mono-secondary`].includes(e);return(0,i.jsx)((0,a.useMemo)(()=>d||(e===`heading1`?`h1`:e===`heading2`?`h2`:e===`heading3`?`h3`:[`mono`,`mono-secondary`].includes(e)?`span`:`p`),[e,d]),{ref:p,className:r(`text-kumo-default`,o.variant[e].classes,m?o.size[n].classes:``,m&&t?`font-medium`:``,h&&(n===`lg`?o.size.base.classes:o.size.sm.classes),s&&`truncate min-w-0`,l),style:u,...f,children:c})}var c=(0,a.forwardRef)(s);export{c as t};