var p=Object.defineProperty;var l=Object.getOwnPropertyDescriptor;var m=Object.getOwnPropertyNames;var r=Object.prototype.hasOwnProperty;var u=(a,t)=>{for(var n in t)p(a,n,{get:t[n],enumerable:!0})},c=(a,t,n,b)=>{if(t&&typeof t=="object"||typeof t=="function")for(let e of m(t))!r.call(a,e)&&e!==n&&p(a,e,{get:()=>t[e],enumerable:!(b=l(t,e))||b.enumerable});return a};var d=a=>c(p({},"__esModule",{value:!0}),a);var h={};u(h,{default:()=>g});module.exports=d(h);var g=[{type:"var",match:/(("|')((?!\2)[^\r\n\\]|\\[^])*\2|[a-zA-Z]\w*)(?=\s*:)/g},{expand:"str"},{expand:"num"},{type:"num",match:/\bnull\b/g},{type:"bool",match:/\b(true|false)\b/g}];
