// src/workers/pipelines/pipeline.worker.ts
import { WorkerEntrypoint } from "cloudflare:workers";
var Pipeline = class extends WorkerEntrypoint {
  async send(_data) {
  }
};
export {
  Pipeline as default
};
//# sourceMappingURL=pipeline.worker.js.map
