#!/usr/bin/env node
const { Log } = require(".");

const log = new Log();
log.error(
	[
		"`miniflare@3` no longer includes a CLI. Please use `npx wrangler dev` instead.",
		"As of `wrangler@3`, this will use Miniflare by default.",
		"See https://miniflare.dev/get-started/migrating for more details.",
	].join("\n")
);
