export declare const getBuiltinModule: NodeJS.Process["getBuiltinModule"];
export declare const exit: {
    (code?: number | string | null | undefined): never;
    (code?: number | string | null | undefined): never;
}, features: NodeJS.ProcessFeatures, platform: NodeJS.Platform;
export declare const _channel: any, _debugEnd: any, _debugProcess: any, _disconnect: any, _events: any, _eventsCount: any, _exiting: any, _fatalException: any, _getActiveHandles: any, _getActiveRequests: any, _handleQueue: any, _kill: any, _linkedBinding: any, _maxListeners: any, _pendingMessage: any, _preload_modules: any, _rawDebug: any, _send: any, _startProfilerIdleNotifier: any, _stopProfilerIdleNotifier: any, _tickCallback: any, abort: any, addListener: any, allowedNodeEnvironmentFlags: any, arch: any, argv: any, argv0: any, assert: any, availableMemory: any, binding: any, channel: any, chdir: any, config: any, connected: any, constrainedMemory: any, cpuUsage: any, cwd: any, debugPort: any, disconnect: any, dlopen: any, domain: any, emit: any, emitWarning: any, env: any, eventNames: any, execArgv: any, execPath: any, exitCode: any, finalization: any, getActiveResourcesInfo: any, getegid: any, geteuid: any, getgid: any, getgroups: any, getMaxListeners: any, getuid: any, hasUncaughtExceptionCaptureCallback: any, hrtime: any, initgroups: any, kill: any, listenerCount: any, listeners: any, loadEnvFile: any, mainModule: any, memoryUsage: any, moduleLoadList: any, nextTick: any, off: any, on: any, once: any, openStdin: any, permission: any, pid: any, ppid: any, prependListener: any, prependOnceListener: any, rawListeners: any, reallyExit: any, ref: any, release: any, removeAllListeners: any, removeListener: any, report: any, resourceUsage: any, send: any, setegid: any, seteuid: any, setgid: any, setgroups: any, setMaxListeners: any, setSourceMapsEnabled: any, setuid: any, setUncaughtExceptionCaptureCallback: any, sourceMapsEnabled: any, stderr: any, stdin: any, stdout: any, throwDeprecation: any, title: any, traceDeprecation: any, umask: any, unref: any, uptime: any, version: any, versions: any;
declare const _default: NodeJS.Process;
export default _default;
