/**
 * @poppinss/colors
 *
 * (c) Poppinss
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
import { type Colors } from './base.ts';
export type { Colors } from './base.ts';
export type ColorTransformations = keyof Colors;
