import { notImplementedClass } from "../_internal/utils.mjs";
export const DatabaseSync = /* @__PURE__ */ notImplementedClass("sqlite.DatabaseSync");
export const StatementSync = /* @__PURE__ */ notImplementedClass("sqlite.StatementSync");
export const constants = {};
export default {
	DatabaseSync,
	StatementSync,
	constants
};
