import{c as e}from"./useRouterState-DyVgzXuA.js";import{St as t}from"./button-cdxnqcgzwko8ooha-DePtDphh.js";var n=e();function r(e={}){return t(`flex w-full flex-col overflow-hidden rounded-lg bg-kumo-elevated text-base ring ring-kumo-line`)}function i({children:e,className:i}){return(0,n.jsx)(`div`,{className:t(r(),i),children:e})}function a({children:e,className:r}){return(0,n.jsx)(`div`,{className:t(`flex items-center gap-2 p-4 text-base font-medium text-kumo-strong -my-2 bg-kumo-elevated`,r),children:e})}function o({children:e,className:r}){return(0,n.jsx)(`div`,{className:t(`relative flex flex-col gap-2 overflow-hidden rounded-lg bg-kumo-base p-4 pr-3 text-inherit no-underline ring ring-kumo-fill`,r),children:e})}var s=Object.assign(i,{Primary:o,Secondary:a});export{s as t};