declare namespace _default {
    let black: string;
    let red: string;
    let green: string;
    let gray: string;
    let yellow: string;
    let blue: string;
    let magenta: string;
    let cyan: string;
    let white: string;
}
export default _default;
//# sourceMappingURL=termcolor.d.ts.map