"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var hash_fn_1 = require("./hash-fn");
exports.hash = hash_fn_1.hash;
exports.deriveKey = hash_fn_1.deriveKey;
exports.keyedHash = hash_fn_1.keyedHash;
__export(require("./hash-instance"));
__export(require("../base/index"));
//# sourceMappingURL=index.js.map