import { type TypeName } from '@sindresorhus/is';
import type { Tokenizer } from '../types.js';
/**
 * Tokenizers are responsible for converting JS data types
 * to known dumper tokens.
 */
export declare const tokenizers: Partial<Record<TypeName, Tokenizer>>;
