import { notImplemented } from "../../_internal/utils.mjs";
export const finished = /* @__PURE__ */ notImplemented("stream.promises.finished");
export const pipeline = /* @__PURE__ */ notImplemented("stream.promises.pipeline");
export default {
	finished,
	pipeline
};
