import{c as e,s as t,z as n}from"./useRouterState-DyVgzXuA.js";import{St as r,t as i}from"./button-cdxnqcgzwko8ooha-DePtDphh.js";import{S as a}from"./api-CJ3Zq7MY.js";import{t as o}from"./Copy.es-87yIntv6.js";var s=n(t(),1),c=e();function l({text:e}){let[t,n]=(0,s.useState)(!1),l=async()=>{await navigator.clipboard.writeText(e),n(!0),setTimeout(()=>n(!1),1500)};return(0,c.jsx)(i,{className:r(`h-6 w-6 p-0 opacity-0 transition-[opacity,background-color,color] group-hover/cell:opacity-100`,{"text-kumo-success opacity-100":t}),onClick:l,"aria-label":t?`Copied`:`Copy to clipboard`,variant:`ghost`,shape:`square`,children:t?(0,c.jsx)(a,{size:14,weight:`bold`}):(0,c.jsx)(o,{size:14})})}export{l as t};